import phonopy
from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
import numpy as np
import os
import su2rot
import sys
import spglib as spg 
from phonopy.structure.symmetry import Symmetry 
from phonopy.interface.vasp import write_vasp
from phonopy.phonon.irreps import IrReps
from phonopy.units import VaspToTHz
from phonopy.structure.atoms import PhonopyAtoms as Atoms
from phonopy.structure.cells import (
    Primitive,
    get_primitive,
    get_primitive_matrix,
    get_supercell,
    guess_primitive_matrix,
    shape_supercell_matrix,
)

'''
Phonopy2trace version:  2022.04.20
Python version: 3.10.4 
Phonopy version: Please use the phonopy package in this folder !!!
Author: Yuanfeng Xu (Princeton University, Zhejiang University)
Contact: xyfphy@gmail.com
Ref. arxiv:2211.11776 (2022)
'''
print('*******************************************************************')
print('Phonopy2trace: version 2022.04.20')
print('Python version: 3.10.4 ')
print('Phonopy version: Please use the phonopy package in this folder !!!')
print('Author: Yuanfeng Xu')
print('Contact: xyfphy@gmail.com')
print('When you use this tool and the \'Check Topological Phonons\' on https://www.cryst.ehu.es/cgi-bin/cryst/programs/magnetictopo.pl?tipog=phonon, please cite the reference: Yuanfeng Xu, M.G. Vergniory, Da-Shuai Ma, Juan L. Mañes, Zhi-Da Song, B. Andrei Bernevig, Nicolas Regnault, Luis Elcoro "Catalogue of topological phonon materials" arxiv:2211.11776 (2022)')
print('*******************************************************************\n')

if len(sys.argv) != 2:
    print('Please type the following commond:')
    print('python phonopy2trace.py 0')
    print('for non-NAC calculation, and ')
    print('python phonopy2trace.py 1')
    print('for NAC calculation.\n')

if sys.argv[1] == '1':
    NAC_tag= True
else:
    NAC_tag= False

print('NAC =',NAC_tag)

dirname=os.getcwd()
#factor=VaspToTHz*4.136 # [meV]
factor=4.136 # [meV]



def append_band(bands, q_start, q_end):
    band = []
    Nk=1
    for i in range(Nk+1):
        band.append(np.array(q_start) +
                    (np.array(q_end) - np.array(q_start)) / Nk * i)
    bands.append(band)


def similarity_transformation(rot, mat):
    """ R x M x R^-1 """
    return np.dot(rot, np.dot(mat, np.linalg.inv(rot)))

def findlg(rk,R):
    tag=[]
    for i in range(len(rk)):        
        for j in range(len(R)):
            log=0
            for l in range(3):
                for m in range(3):
                    log+=abs(R[j,l,m]-rk[i,l,m])
            if log < 1e-3:
                tag.append(j+1)
                break
    return tag        

if __name__ == '__main__':
    # the materal list to run
    # format for each line: SG SG_symbol mpid ChemicalFormula
    fo=open('materials.txt','r')
    
    for mat_ in fo.readlines():
        mat_=mat_.strip('\n').split()
        mpid=mat_[2]
        formula=mat_[3]
        SG=mat_[0]
        if NAC_tag:
            tracename=SG+'_'+formula+'_phonondataNAC_'+mpid+'-mpid.txt'
        else:
            tracename=SG+'_'+formula+'_phonondata_'+mpid+'-mpid.txt'
        print('mpid:',mpid)
        os.chdir(dirname+'/'+mpid)
        cell_ = read_vasp("POSCAR-unitcell")
    
    #supercell shape and primitive_matrix from phonopy.conf file
        scsize=np.zeros((3,3),dtype=np.int64)
        primitive_matrix_=np.diag([1.0, 1.0, 1.0]) #((3,3),dtype=np.complex128)
        f=open('phonopy.conf','r')
        lines=f.readlines()
        for i in range(3):
            for j in range(3):
                scsize[i,j]=int(lines[0].split()[i*3+j+2])
                if len(lines)==2:
                    primitive_matrix_[i,j]=float(eval(lines[1].split()[i*3+j+2]+'.0'))
        phonon = Phonopy(cell_,scsize,primitive_matrix_)
        cell1 = Primitive(cell_, primitive_matrix_, symprec=1e-4)
        rprim = cell1.cell #* distance_to_A
#       print(rprim)
        positions = cell1.scaled_positions
        #***********************
        #Symmetry
        #***********************
        symmetry = phonon.get_symmetry()
        print ("Space group:", symmetry.get_international_table())
        spg_symbol, spg_number = symmetry.get_international_table().split()
        spg_number = int(spg_number.replace('(', '').replace(')', ''))
#       print(spg_number, spg_symbol)
        prim_symm = phonon.get_primitive_symmetry()
        rotation = prim_symm.get_symmetry_operations()['rotations']
        translation = prim_symm.get_symmetry_operations()['translations']
        nsym=len(rotation)
        SU2=np.zeros((nsym,2,2),dtype=np.complex128)
        for i in range(nsym):
            sopr_c=similarity_transformation(np.transpose(rprim),rotation[i,:,:])
            SU2[i,:,:]=su2rot.get_su2rotation(sopr_c)
        
        #****************
        #max-kpoits
        #****************
        with open('../max_KPOINTS_VASP/KPOINTS_'+str(spg_number)+'.txt','r') as f:
            line=f.readline()
            line=f.readline().split()
            nsymk=int(eval(line[0]))
            symk =np.zeros((nsymk,3),dtype=np.float64)
            line=f.readline()
            for i in range(nsymk):
                line=f.readline().split()
                for j in range(3):
                    symk[i,j]=eval(line[j])
                if abs(symk[i,0])<1e-6 and abs(symk[i,1])<1e-6 and abs(symk[i,2])<1e-6:
                    gammak=i
        
        #************************
        #Force file
        #************************
    #   force_sets = parse_FORCE_SETS()
    #   phonon.set_displacement_dataset(force_sets)
    #   phonon.produce_force_constants()
        phonon = phonopy.load("phonon.yaml",primitive_matrix=primitive_matrix_,symmetrize_fc=True,is_nac=NAC_tag)
        dmat = phonon.get_dynamical_matrix_at_q([0,0,0])
        Nband,Nband1=dmat.shape
        Natom=int(Nband/3.0)
        ft=open(tracename,'w')
        ft.write(str(Nband)+'\n')
        ft.write('0\n')
        ft.write(str(nsym)+'\n')
        for i in range(nsym):
            for ii in range(3):
                for jj in range(3):
                    ft.write('{:3d}'.format((rotation[i,ii,jj])))
            for ii in range(3):
                if abs(translation[i,ii]-1.0)<1e-3:
                    translation[i,ii]=0.0
                ft.write('{:10.5f}'.format(translation[i,ii]))
            for ii in range(2):
                for jj in range(2):
                    ft.write('{:14.7f}{:14.7f}'.format(SU2[i,ii,jj].real,SU2[i,ii,jj].imag))
            ft.write('\n')
        ft.write(str(nsymk))
        ft.write('\n')
        for ik in range(nsymk):
            for ii in range(3):
                ft.write('{:14.7}'.format(symk[ik,ii]))
            ft.write('\n')
        # Example to obtain dynamical matrix
        for ik in range(nsymk):
            kpt=symk[ik]
         #   little group
            rk,tk=phonon.set_irreps1(symk[ik], 1e-4)
            setirep=phonon.set_irreps(symk[ik], 1e-2)
            ct = phonon.get_irreps()
            lgtag=findlg(rk,rotation)
            ft.write(str(len(lgtag))+'\n')
            for opr in lgtag:
                ft.write(str(opr)+'  ')
            ft.write('\n')
        #   phonon.show_irreps()
            band_indices = ct.get_band_indices()
            ek1 = ct.get_freq()
            ek = []
            for eig in ek1:
                ek.append(eig*factor)
        #   characters = np.rint(ct.get_characters())#.real
            characters = ct.get_characters()
            for bi, cts in zip(band_indices, characters):
                ft.write('{:5s}{:3s}{:12.6f}'.format(str(bi[0]+1),str(len(bi)),ek[bi[0]]))
                for iopr in range(len(lgtag)):
                    ft.write('{:10.5f}{:10.5f}'.format(cts[iopr].real ,cts[iopr].imag))
                ft.write('\n')
        ft.close()
        os.chdir(dirname)
    
